import 'dart:async';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:lottie/lottie.dart';
import 'package:_private_core/_private_core.dart';

class WidgetCallComming extends StatelessWidget {
  final String name;
  final Function()? onCancel;
  final Function()? onAccept;
  const WidgetCallComming({
    Key? key,
    required this.name,
    required this.onCancel,
    required this.onAccept,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
          color: Colors.white, borderRadius: BorderRadius.circular(16)),
      child: Column(
        children: [
          Container(
              margin: const EdgeInsets.only(top: 30),
              padding: const EdgeInsets.symmetric(horizontal: 50, vertical: 12),
              decoration: BoxDecoration(
                  color: Colors.white30,
                  borderRadius: BorderRadius.circular(15)),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  Text(name, style: robotoBold),
                  const SizedBox(
                    height: 4,
                  ),
                  Text('00:00:00', style: robotoBold),
                ],
              )),
          Expanded(
              child: Center(
            child: SizedBox(
              width: 250,
              height: 250,
              child: Stack(
                alignment: Alignment.center,
                children: [
                  Lottie.asset(assetlottie('incomingCall'),
                      package: '_private_chat', width: 250, height: 250),
                  const Icon(
                    Icons.videocam,
                    color: Colors.white,
                    size: 42.0,
                  )
                ],
              ),
            ),
          )),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  GestureDetector(
                    onTap: onCancel,
                    child: Container(
                      padding: const EdgeInsets.all(15.0),
                      margin: const EdgeInsets.symmetric(
                          horizontal: 20, vertical: 10),
                      decoration: BoxDecoration(
                        shape: BoxShape.circle,
                        color: Colors.white,
                      ),
                      child: Icon(
                        Icons.close,
                        size: 35,
                        color: appColors.text,
                      ),
                    ),
                  ),
                  Text(
                    'cancel'.tr,
                    style: robotoBold.copyWith(color: Colors.white),
                  )
                ],
              ),
              Column(
                children: [
                  BlinkCallButtonWidget(
                    onTap: onAccept,
                  ),
                  Text(
                    'start'.tr,
                    style: robotoBold.copyWith(color: Colors.green),
                  )
                ],
              ),
            ],
          ),
          const SizedBox(
            height: 20.0,
          )
        ],
      ),
    );
  }
}

class BlinkCallButtonWidget extends StatefulWidget {
  final Function()? onTap;

  const BlinkCallButtonWidget({Key? key, required this.onTap})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => _BlinkCallButtonWidget();
}

class _BlinkCallButtonWidget extends State<BlinkCallButtonWidget> {
  Timer? _timer;
  bool _show = true;

  @override
  void initState() {
    super.initState();
    _loop();
  }

  _loop() {
    _timer = Timer(const Duration(milliseconds: 500), () {
      setState(() {
        _show = !_show;
      });
      _loop();
    });
  }

  @override
  void dispose() {
    super.dispose();
    _timer?.cancel();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: widget.onTap,
      child: Container(
        padding: const EdgeInsets.all(15.0),
        margin: const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
        decoration: BoxDecoration(
          shape: BoxShape.circle,
          color: _show ? Colors.green : Colors.transparent,
        ),
        child: Icon(
          Icons.call,
          size: 35,
          color: appColors.text,
        ),
      ),
    );
  }
}
