import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/widgets.dart';

import 'package:flutter/material.dart';
import 'package:_private_core/widgets/widgets.dart';

const Duration messageAnimationDuration = Duration(milliseconds: 350);
const Duration messageReverseAnimationDuration = Duration(milliseconds: 150);

enum MessageIndexOfGroup { top, bottom, center, none }

enum MessageType {
  text,
  image,
  video,
  doc,
  location,
  contact,
  audio,
  gif,
  homelido_appointment,
  lottie_emoji,
}

class Message {
  Message({
    required this.widgetMessageOwner,
    required this.timestamp,
    required this.animationController,
    required this.snappableKey,
    this.doc,
    this.tempDoc,
  });

  final Widget Function(List<Message>) widgetMessageOwner;
  final int? timestamp;
  final AnimationController animationController;
  final GlobalKey<SnappableState> snappableKey;
  Map<String, dynamic>? tempDoc;
  DocumentSnapshot? doc;

  static Widget wrapMessage({
    required dynamic child,
    required AnimationController animationController,
    required keySnappableState,
  }) {
    return SizeTransition(
      sizeFactor:
          CurvedAnimation(parent: animationController, curve: Curves.easeOut),
      axisAlignment: 0.0,
      child: Snappable(
        key: keySnappableState,
        numberOfBuckets: 16,
        // offset: isMe ? const Offset(120, 80) : const Offset(-120, 80),
        offset: const Offset(120, 80),
        randomDislocationOffset: const Offset(100, 100),
        duration: const Duration(milliseconds: 2000),
        child: child,
      ),
    );
  }
}

///Seen state
class SeenState extends ValueNotifier<Map<String, dynamic>> {
  SeenState(value) : super(value);
}

class SeenProvider extends StatefulWidget {
  const SeenProvider({this.timestamp, this.seen, this.reaction, this.child});
  final SeenState? seen;
  final ReactionState? reaction;
  final Widget? child;
  final String? timestamp;

  static of(BuildContext context) {
    _SeenInheritedProvider? p = context.dependOnInheritedWidgetOfExactType(
        aspect: _SeenInheritedProvider);
    return p!.data;
  }

  @override
  State<StatefulWidget> createState() => _SeenProviderState();
}

class _SeenProviderState extends State<SeenProvider> {
  @override
  initState() {
    super.initState();
    widget.seen!.addListener(didValueChange);
  }

  didValueChange() {
    if (mounted) setState(() {});
  }

  @override
  dispose() {
    widget.seen!.removeListener(didValueChange);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _SeenInheritedProvider(
      data: widget.seen,
      child: ReactionProvider(
        timestamp: widget.timestamp,
        reaction: widget.reaction,
        child: widget.child!,
      ),
    );
  }
}

class _SeenInheritedProvider extends InheritedWidget {
  _SeenInheritedProvider({required this.data, required this.child})
      : _dataValue = data.value,
        super(child: child);
  final data;
  final Widget child;
  final _dataValue;
  @override
  bool updateShouldNotify(_SeenInheritedProvider oldWidget) {
    return _dataValue != oldWidget._dataValue;
  }
}

///Reaction state

class ReactionState extends ValueNotifier {
  ReactionState(value) : super(value);
}

class ReactionProvider extends StatefulWidget {
  const ReactionProvider({this.timestamp, this.reaction, this.child});
  final ReactionState? reaction;
  final Widget? child;
  final String? timestamp;

  static of(BuildContext context) {
    _ReactionInheritedProvider? p = context.dependOnInheritedWidgetOfExactType(
        aspect: _ReactionInheritedProvider);
    return p!.data;
  }

  @override
  State<StatefulWidget> createState() => new _ReactionProviderState();
}

class _ReactionProviderState extends State<ReactionProvider> {
  @override
  initState() {
    super.initState();
    widget.reaction!.addListener(didValueChange);
  }

  didValueChange() {
    if (mounted) this.setState(() {});
  }

  @override
  dispose() {
    widget.reaction!.removeListener(didValueChange);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return new _ReactionInheritedProvider(
      data: widget.reaction,
      child: widget.child!,
    );
  }
}

class _ReactionInheritedProvider extends InheritedWidget {
  _ReactionInheritedProvider({required this.data, required this.child})
      : _dataValue = data.value,
        super(child: child);
  final data;
  final Widget child;
  final _dataValue;
  @override
  bool updateShouldNotify(_ReactionInheritedProvider oldWidget) {
    return _dataValue != oldWidget._dataValue;
  }
}
