import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:lottie/lottie.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core/_private_core.dart';
import 'package:_private_core/widgets/widgets.dart';

class WidgetHomelidoAppointmentMessage extends StatefulWidget {
  final dynamic meetingId;
  const WidgetHomelidoAppointmentMessage({Key? key, required this.meetingId})
      : super(key: key);

  @override
  _WidgetHomelidoAppointmentMessageState createState() =>
      _WidgetHomelidoAppointmentMessageState();
}

class _WidgetHomelidoAppointmentMessageState
    extends State<WidgetHomelidoAppointmentMessage> {
  final HomelidoRepo repo = HomelidoRepo();
  final key = GlobalKey<ExpandableNotifierState>();
  bool isShowAssit = false;
  HomelidoMeeting? m;

  Color get textColor => appColors.text;
  Color get dividerColor => hexColor('#3472D1');
  Color get cancelColor => hexColor('#E65A7B');
  Color get acceptColor => hexColor('#8FD597');
  Color get bubbleColor => appColors.appBGColor;

  Future<HomelidoMeeting?> _getDetail() async {
    if (key.currentState?.controller?.expanded == true) {
      m = await repo.getHomelidoMeetingDetails(meetingId: widget.meetingId);
    }
    return m;
  }

  _onDecline() async {
    await repo.answerToHomelidoVisitRequest(
        meetingId: widget.meetingId, status: HOMELIDO_MEETING_STATUS_DECLINED);
    setState(() {});
  }

  _onAccept() async {
    final r = await repo.answerToHomelidoVisitRequest(
        meetingId: widget.meetingId, status: HOMELIDO_MEETING_STATUS_ACCEPTED);
    if (r) {
      setState(() {});
      chatOptions!.chatonremovelocalhomelidocard
          ?.call(m?.homelidoPropertyInfo?.id);
    }
  }

  Widget get iconMeeting => Image.asset(
        'assets/images/png/icmeeting.png',
        width: 20,
        height: 20,
      );
  Widget get iconEvent => Image.asset(
        'assets/images/png/iccalendar.png',
        width: 20,
        height: 20,
      );
  Widget get iconLocation => Image.asset(
        'assets/images/png/iclocationpoint.png',
        width: 20,
        height: 20,
      );
  Widget get iconArea => Image.asset(
        'assets/images/png/icarea.png',
        width: 20,
        height: 20,
      );

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(color: mainColor),
      child: ExpandableNotifier(
        key: key,
        initialExpanded: false,
        child: ExpandablePanel(
          theme: ExpandableThemeData(
            headerAlignment: ExpandablePanelHeaderAlignment.center,
            tapBodyToExpand: false,
            tapBodyToCollapse: false,
            hasIcon: false,
          ),
          collapsed: GestureDetector(
            onTap: () {
              key.currentState!.controller!.toggle();
              setState(() {});
            },
            child: Container(
              color: Colors.transparent,
              padding: const EdgeInsets.symmetric(horizontal: 24, vertical: 14),
              child: Row(
                children: [
                  iconMeeting,
                  const SizedBox(
                    width: 12,
                  ),
                  Expanded(
                    child: Text(
                      'meetingproposition'.tr,
                      style: robotoBold.copyWith(color: textColor),
                    ),
                  ),
                ],
              ),
            ),
          ),
          expanded: Padding(
            padding: const EdgeInsets.symmetric(vertical: 12, horizontal: 24),
            child: FutureBuilder<HomelidoMeeting?>(
                future: _getDetail(),
                builder: (_, snapshot) {
                  Widget child = const SizedBox();
                  HomelidoMeeting m = snapshot.data ??
                      HomelidoMeeting(status: HOMELIDO_MEETING_STATUS_WAITING);
                  if (snapshot.data != null) {
                    child = Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        const SizedBox(
                          height: 12,
                        ),
                        Container(
                          color: Colors.transparent,
                          padding: const EdgeInsets.symmetric(vertical: 12),
                          child: Row(
                            children: [
                              iconEvent,
                              Expanded(
                                child: Padding(
                                  padding: const EdgeInsets.only(left: 52),
                                  child: Text(
                                    DateTime.fromMillisecondsSinceEpoch(
                                            m.meetingDatetime! * 1000)
                                        .formatDateTime(),
                                    style: robotoBold.copyWith(
                                        fontSize: 13.5,
                                        fontWeight: FontWeight.w500,
                                        color: Colors.white),
                                  ),
                                ),
                              ),
                            ],
                          ),
                        ),
                        AnimatedCrossFade(
                          duration: const Duration(milliseconds: 350),
                          crossFadeState: isShowAssit
                              ? CrossFadeState.showSecond
                              : CrossFadeState.showFirst,
                          firstChild: SizedBox(
                            width: Get.width,
                          ),
                          secondChild: _buildBubbleMessage(true),
                        ),
                        Container(
                          height: 1,
                          color: dividerColor,
                          width: Get.width,
                        ),
                        Container(
                          color: Colors.transparent,
                          padding: const EdgeInsets.symmetric(vertical: 12),
                          child: Row(
                            children: [
                              iconLocation,
                              Expanded(
                                child: Padding(
                                  padding: const EdgeInsets.only(left: 52),
                                  child: Text(
                                    m.homelidoPropertyInfo?.address ?? '',
                                    maxLines: 1,
                                    overflow: TextOverflow.ellipsis,
                                    style: robotoBold.copyWith(
                                        fontWeight: FontWeight.w500,
                                        fontSize: 13.5,
                                        color: Colors.white),
                                  ),
                                ),
                              ),
                            ],
                          ),
                        ),
                        AnimatedCrossFade(
                          duration: const Duration(milliseconds: 350),
                          crossFadeState: isShowAssit
                              ? CrossFadeState.showSecond
                              : CrossFadeState.showFirst,
                          firstChild: SizedBox(
                            width: Get.width,
                          ),
                          secondChild: _buildBubbleMessage(false),
                        ),
                        Container(
                          height: 1,
                          color: dividerColor,
                          width: Get.width,
                        ),
                        Container(
                          padding: const EdgeInsets.fromLTRB(0, 16, 0, 8),
                          child: Row(
                            children: [
                              WidgetAppImage(
                                imageUrl: appImageCorrectUrl(
                                    m.homelidoPropertyInfo?.image ?? '',
                                    base: arckipelbaseUrlHomelido),
                                width: 105,
                                height: 75,
                                boxFit: BoxFit.cover,
                                radius: 12,
                              ),
                              const SizedBox(
                                width: 20,
                              ),
                              Expanded(
                                child: SizedBox(
                                  height: 80,
                                  child: Column(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: [
                                      Text(
                                        m.homelidoPropertyInfo?.title ?? '',
                                        maxLines: 1,
                                        overflow: TextOverflow.ellipsis,
                                        style: robotoBold.copyWith(
                                          color: Colors.white,
                                          fontSize: 14.5,
                                        ),
                                      ),
                                      const SizedBox(
                                        height: 6,
                                      ),
                                      Text(
                                        m.homelidoPropertyInfo?.salePrice !=
                                                null
                                            ? '${m.homelidoPropertyInfo?.salePrice} ${currencySymbol(m.homelidoPropertyInfo?.currency)}'
                                            : '${m.homelidoPropertyInfo?.price} ${currencySymbol(m.homelidoPropertyInfo?.currency)}/${'month'.tr}',
                                        style: textStyleSmall.copyWith(
                                            color: appColors.redHomelido,
                                            fontSize: 12),
                                      ),
                                      const SizedBox(
                                        height: 6,
                                      ),
                                      Row(
                                        crossAxisAlignment:
                                            CrossAxisAlignment.end,
                                        children: [
                                          iconArea,
                                          const SizedBox(
                                            width: 4,
                                          ),
                                          Text(
                                            m.homelidoPropertyInfo?.area ==
                                                        null ||
                                                    m.homelidoPropertyInfo
                                                            ?.areaUnit ==
                                                        null
                                                ? 'unknow'.tr
                                                : '${m.homelidoPropertyInfo?.area} ${m.homelidoPropertyInfo?.areaUnit}',
                                            style: textStyleSmall.copyWith(
                                                color: Colors.white,
                                                fontSize: 9),
                                          ),
                                          const SizedBox(
                                            width: 24,
                                          ),
                                          const Icon(
                                            Icons.airline_seat_individual_suite,
                                            size: 14,
                                            color: Colors.white,
                                          ),
                                          const SizedBox(
                                            width: 4,
                                          ),
                                          Text(
                                            m.homelidoPropertyInfo?.bed == null
                                                ? 'unknow'.tr
                                                : '${m.homelidoPropertyInfo?.bed} ${'bedrooms'.tr}',
                                            style: textStyleSmall.copyWith(
                                                color: Colors.white,
                                                fontSize: 9),
                                          ),
                                          const Spacer(),
                                          InkWell(
                                            onTap: () {
                                              // showMaterialModalBottomSheet(
                                              //     backgroundColor:
                                              //         Colors.transparent,
                                              //     context: Get.context!,
                                              //     builder: (_) => HomelidoScreen(
                                              //         m: m.homelidoPropertyInfo!));
                                            },
                                            child: Text(
                                              'more...'.tr,
                                              style: textStyleSmall.copyWith(
                                                  color: textColor,
                                                  fontSize: 8.5),
                                            ),
                                          ),
                                        ],
                                      )
                                    ],
                                  ),
                                ),
                              )
                            ],
                          ),
                        ),
                        if (m.status == HOMELIDO_MEETING_STATUS_WAITING &&
                            appPrefs.loginUser.isUserVersion)
                          Container(
                            padding: EdgeInsets.fromLTRB(0, 8, 0,
                                16 + MediaQuery.of(context).padding.bottom),
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.spaceBetween,
                              children: [
                                GestureDetector(
                                  onTap: _onDecline,
                                  child: Container(
                                    width: Get.width / 2.75,
                                    decoration: BoxDecoration(
                                        color: appColors.appBGColor,
                                        border: Border.all(
                                            color: appColors.appBGColor,
                                            width: 1.2),
                                        borderRadius:
                                            BorderRadius.circular(12)),
                                    padding: const EdgeInsets.symmetric(
                                        vertical: 10),
                                    child: Center(
                                      child: Text(
                                        'decline'.tr,
                                        style: robotoBold.copyWith(
                                            color: textColor),
                                      ),
                                    ),
                                  ),
                                ),
                                GestureDetector(
                                  onTap: _onAccept,
                                  child: Container(
                                    width: Get.width / 2.75,
                                    decoration: BoxDecoration(
                                        color: Colors.white,
                                        border: Border.all(
                                            color: textColor, width: 1.2),
                                        borderRadius:
                                            BorderRadius.circular(12)),
                                    padding: const EdgeInsets.symmetric(
                                        vertical: 10),
                                    child: Center(
                                      child: Text(
                                        'accept'.tr,
                                        style: robotoBold.copyWith(
                                            color: Colors.black),
                                      ),
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          ),
                        if (m.status == HOMELIDO_MEETING_STATUS_ACCEPTED)
                          Column(
                            children: [
                              Container(
                                height: 1,
                                color: dividerColor,
                                width: Get.width,
                              ),
                              Padding(
                                padding:
                                    const EdgeInsets.symmetric(vertical: 10),
                                child: Row(
                                  children: [
                                    Icon(
                                      Icons.check_circle,
                                      color: textColor,
                                    ),
                                    const SizedBox(
                                      width: 16,
                                    ),
                                    Text(
                                      'accepted'.tr,
                                      style: robotoMedium.copyWith(
                                          fontSize: 13.5, color: acceptColor),
                                    )
                                  ],
                                ),
                              ),
                              Container(
                                height: 1,
                                color: dividerColor,
                                width: Get.width,
                              ),
                              const SizedBox(
                                height: 16,
                              ),
                              if (chatOptions!.chatopeninmycalendar != null)
                                GestureDetector(
                                  onTap: () {
                                    chatOptions!
                                        .chatopeninmycalendar(widget.meetingId);
                                  },
                                  child: Container(
                                    width: Get.width,
                                    decoration: BoxDecoration(
                                        color: textColor,
                                        border: Border.all(
                                            color: textColor, width: 1.2),
                                        borderRadius:
                                            BorderRadius.circular(12)),
                                    padding: const EdgeInsets.symmetric(
                                        vertical: 10),
                                    child: Center(
                                      child: Text(
                                        'openinmycalendar'.tr,
                                        style: robotoBold.copyWith(
                                            color: appColors.appBGColor),
                                      ),
                                    ),
                                  ),
                                ),
                              const SizedBox(
                                height: 8,
                              ),
                            ],
                          ),
                        if (m.status ==
                                HOMELIDO_MEETING_STATUS_CANCELED_BY_USER ||
                            m.status == HOMELIDO_MEETING_STATUS_CANCELED_BY_PRO)
                          Column(
                            children: [
                              Container(
                                height: 1,
                                color: dividerColor,
                                width: Get.width,
                              ),
                              Padding(
                                padding:
                                    const EdgeInsets.symmetric(vertical: 10),
                                child: Row(
                                  children: [
                                    Icon(
                                      Icons.cancel,
                                      color: textColor,
                                    ),
                                    const SizedBox(
                                      width: 16,
                                    ),
                                    Text(
                                      'cancelled'.tr,
                                      style: robotoMedium.copyWith(
                                          fontSize: 13.5, color: cancelColor),
                                    )
                                  ],
                                ),
                              ),
                              Container(
                                height: 1,
                                color: dividerColor,
                                width: Get.width,
                              ),
                              const SizedBox(
                                height: 16,
                              ),
                              if (chatOptions!.chatopeninmycalendar != null)
                                GestureDetector(
                                  onTap: () {
                                    chatOptions!
                                        .chatopeninmycalendar(widget.meetingId);
                                  },
                                  child: Container(
                                    width: Get.width,
                                    decoration: BoxDecoration(
                                        color: textColor,
                                        border: Border.all(
                                            color: textColor, width: 1.2),
                                        borderRadius:
                                            BorderRadius.circular(12)),
                                    padding: const EdgeInsets.symmetric(
                                        vertical: 10),
                                    child: Center(
                                      child: Text(
                                        'openinmycalendar'.tr,
                                        style: robotoBold.copyWith(
                                            color: Colors.white),
                                      ),
                                    ),
                                  ),
                                ),
                              const SizedBox(
                                height: 8,
                              ),
                            ],
                          ),
                        if (m.status == HOMELIDO_MEETING_STATUS_DECLINED)
                          Column(
                            children: [
                              Container(
                                height: 1,
                                color: dividerColor,
                                width: Get.width,
                              ),
                              const SizedBox(
                                height: 12,
                              ),
                              Row(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  Icon(
                                    Icons.cancel,
                                    color: textColor,
                                  ),
                                  const SizedBox(
                                    width: 16,
                                  ),
                                  Expanded(
                                    child: Text(
                                      'youhavedeclinedthisinviation'.tr,
                                      style: robotoMedium.copyWith(
                                          fontSize: 13.5, color: cancelColor),
                                    ),
                                  )
                                ],
                              ),
                            ],
                          ),
                      ],
                    );
                  }
                  return Column(
                    children: [
                      GestureDetector(
                        onTap: () {
                          key.currentState!.controller!.toggle();
                        },
                        child: Container(
                          color: Colors.transparent,
                          child: Row(
                            children: [
                              iconMeeting,
                              const SizedBox(
                                width: 12,
                              ),
                              Text(
                                'meetingproposition'.tr,
                                textAlign: TextAlign.center,
                                style: robotoBold.copyWith(color: textColor),
                              ),
                              const Spacer(),
                              if (m.status ==
                                      HOMELIDO_MEETING_STATUS_DECLINED ||
                                  m.status ==
                                      HOMELIDO_MEETING_STATUS_CANCELED_BY_USER ||
                                  m.status ==
                                      HOMELIDO_MEETING_STATUS_CANCELED_BY_PRO)
                                Icon(
                                  Icons.report_problem,
                                  color: cancelColor,
                                  size: 36,
                                )
                              else
                                GestureDetector(
                                  onTap: () {
                                    setState(() {
                                      isShowAssit = !isShowAssit;
                                    });
                                  },
                                  child: Container(
                                    color: Colors.transparent,
                                    child: Column(
                                      children: [
                                        Lottie.asset(
                                          'assets/lotties/assist.json',
                                          width: 36,
                                          height: 36,
                                        ),
                                        const SizedBox(
                                          height: 2,
                                        ),
                                        Text(
                                          'arckipelassistant'.tr,
                                          textAlign: TextAlign.center,
                                          style: robotoMedium.copyWith(
                                              fontSize: 7, color: Colors.white),
                                        ),
                                      ],
                                    ),
                                  ),
                                ),
                            ],
                          ),
                        ),
                      ),
                      child
                    ],
                  );
                }),
          ),
        ),
      ),
    );
  }

  Widget _buildBubbleMessage(bool isDateTime) {
    if (chatOptions!.chatgetDifferentTimePreEventByEventId == null ||
        chatOptions!.chatgetPreEventByEventId == null ||
        chatOptions!.chatgetDistanceBetweenUserAndPlaceByEventId == null) {
      return const SizedBox();
    }
    dynamic c = chatOptions!.chatgetPreEventByEventId(widget.meetingId);
    Duration? d =
        chatOptions!.chatgetDifferentTimePreEventByEventId(widget.meetingId);
    if ((c == null || d == null) && isDateTime) return const SizedBox();
    return Padding(
      padding: const EdgeInsets.fromLTRB(0, 0, 0, 8),
      child: Row(
        children: [
          Lottie.asset(
            'assets/lotties/assist.json',
            width: 52,
            height: 52,
          ),
          SizedBox(
            width: 20,
            child: Align(
              child: ClipPath(
                clipper: _Cliper(),
                child: Container(
                  width: 20,
                  height: 20,
                  decoration: BoxDecoration(color: bubbleColor),
                ),
              ),
            ),
          ),
          Expanded(
            child: Container(
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(16), color: bubbleColor),
              padding: const EdgeInsets.all(12),
              child: isDateTime
                  ? RichText(
                      text: TextSpan(children: [
                      TextSpan(text: 'thepreevent('.tr),
                      TextSpan(
                        text: '${chatOptions!.chatgettitleEvent(c!)}',
                        style: robotoBold.copyWith(
                            color: chatOptions!.chatgetcardColor(c),
                            decoration: TextDecoration.underline),
                        recognizer: TapGestureRecognizer()
                          ..onTap = () {
                            chatOptions!.chatopeninmycalendar(c.type);
                          },
                      ),
                      TextSpan(
                          text: ')inyourcalenadr'.trParams({
                        'hour': '${d!.inHours}',
                        'min': '${d.inMinutes.remainder(60)}'
                      }))
                    ], style: robotoMedium))
                  : FutureBuilder(
                      future: chatOptions!
                          .chatgetDistanceBetweenUserAndPlaceByEventId(
                              m!.homelidoPropertyInfo!.lat,
                              m!.homelidoPropertyInfo!.lng),
                      builder: (_, snap) {
                        if (snap.data == null) return const SizedBox();
                        return Text(
                          'thedistancebetweenyouandthisplace'
                              .trParams({'value': '${snap.data}'}),
                          style: robotoMedium,
                        );
                      }),
            ),
          )
        ],
      ),
    );
  }
}

class _Cliper extends CustomClipper<Path> {
  @override
  Path getClip(Size size) {
    Path path = Path()
      ..moveTo(0, size.height * .7)
      ..quadraticBezierTo(
          size.width / 2, size.height * .95, size.width, size.height)
      ..lineTo(size.width, 0)
      ..quadraticBezierTo(
        size.width / 2,
        size.height * .95,
        0,
        size.height * .7,
      );
    return path;
  }

  @override
  bool shouldReclip(covariant CustomClipper<Path> oldClipper) {
    return true;
  }
}
