import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:get/get.dart' as getx;
import 'package:get/get.dart';
import 'package:_private_chat/utils/utils.dart';
import 'package:_private_core/_private_core.dart';

import '../../chat_list/widget/widget_user_builder.dart';
import '../bloc/chat_detail_bloc.dart';
import '../chat_helper.dart';
import '../message.dart';
import 'widget_audio_record.dart';

class WidgetInputChat extends StatefulWidget {
  final bool isEmbedWidget;
  final Widget Function(
      bool isEmbedWidget,
      dynamic state,
      TextEditingController controller,
      FocusNode node,
      Function() onSend)? builder;
  const WidgetInputChat({
    Key? key,
    this.isEmbedWidget = false,
    this.builder,
  }) : super(key: key);

  @override
  State<WidgetInputChat> createState() => _WidgetInputChatState();
}

class _WidgetInputChatState extends State<WidgetInputChat> {
  late final ChatDetailBloc _chatDetailBloc = Get.find<ChatDetailBloc>();
  TextEditingController get controller => _chatDetailBloc.inputController;
  FocusNode get _inputFocus => _chatDetailBloc.inputFocus;

  Color get colorText => Colors.white;

  Color get chatInputColor => hexColor('#1B1F2E');

  InputBorder get _inputBorder => OutlineInputBorder(
        borderRadius: BorderRadius.circular(1),
        borderSide: const BorderSide(color: Colors.transparent, width: 1.5),
      );

  void onSend() {
    if (controller.text.trim().isNotEmpty) {
      _chatDetailBloc.add(ChatSendMessageEvent(
          type: MessageType.text, text: controller.text.trim()));
      setState(() {
        controller.clear();
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    if (chatOptions!.chatDetailInputBuilder != null || widget.builder != null) {
      return BlocBuilder<ChatDetailBloc, ChatDetailState>(
          bloc: _chatDetailBloc,
          builder: (_, state) {
            return widget.builder?.call(widget.isEmbedWidget, state, controller,
                    _inputFocus, onSend) ??
                chatOptions!.chatDetailInputBuilder!(widget.isEmbedWidget,
                    state, controller, _inputFocus, onSend);
          });
    }
    return Container(
      padding: const EdgeInsets.only(top: 8, bottom: 20, left: 20, right: 20),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          BlocBuilder<ChatDetailBloc, ChatDetailState>(
              bloc: _chatDetailBloc,
              builder: (_, state) {
                if (state.isReplyKeyboard) {
                  Widget child = Text(
                    ChatHelper.decrypt(
                        _chatDetailBloc.conversationData![DB_KEY_ENCRYPT_KEY],
                        state.replyDoc![DB_KEY_CONTENT]),
                    style: textStyleSmall.copyWith(
                        fontSize: 14.0, color: colorText),
                    overflow: TextOverflow.ellipsis,
                    maxLines: 1,
                  );
                  if (state.replyDoc![DB_KEY_MESSAGE_TYPE] ==
                      MessageType.image.index) {
                    child = ClipRRect(
                      borderRadius: BorderRadius.circular(4),
                      child: Image.network(
                        state.replyDoc![DB_KEY_CONTENT_URL],
                        height: 40,
                        fit: BoxFit.contain,
                      ),
                    );
                  }
                  return Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Row(
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          Icon(
                            Icons.reply_rounded,
                            size: 20,
                            color: colorText,
                          ),
                          const SizedBox(
                            width: 4,
                          ),
                          Text(
                            'reply'.tr,
                            style: textStyleSmall.copyWith(
                              color: colorText,
                              fontSize: 14.0,
                            ),
                          ),
                        ],
                      ),
                      const SizedBox(
                        height: 4,
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: hexColor('#131924'),
                            borderRadius: const BorderRadius.horizontal(
                                right: Radius.circular(16))),
                        child: Row(
                          children: [
                            Container(
                              height: 60,
                              width: 4.5,
                              margin: const EdgeInsets.only(right: 8),
                              decoration: BoxDecoration(
                                  color: colorText,
                                  borderRadius: const BorderRadius.horizontal(
                                      right: Radius.circular(12))),
                            ),
                            Expanded(
                              child: Container(
                                padding: const EdgeInsets.only(left: 16),
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: [
                                    WidgetUserChatListBuilder(
                                        id: state.replyDoc![DB_KEY_FROM],
                                        builder: (fullname, thumbnail) {
                                          return Text(
                                            state.replyDoc![DB_KEY_FROM] ==
                                                    loggedFirebaseId
                                                ? 'you'.tr
                                                : fullname,
                                            overflow: TextOverflow.ellipsis,
                                            style: textStyleSmall.copyWith(
                                                fontWeight: FontWeight.bold,
                                                fontSize: 14),
                                          );
                                        }),
                                    const SizedBox(
                                      height: 8,
                                    ),
                                    child
                                  ],
                                ),
                              ),
                            ),
                            const SizedBox(
                              width: 16,
                            ),
                            InkWell(
                              onTap: () {
                                _chatDetailBloc
                                    .add(ChatDetailResetEditOrReplyEvent());
                              },
                              child: Icon(
                                Icons.cancel_rounded,
                                size: 32,
                                color: appColors.element,
                              ),
                            ),
                            const SizedBox(
                              width: 20,
                            ),
                          ],
                        ),
                      ),
                      const SizedBox(
                        height: 12,
                      ),
                    ],
                  );
                }
                return const SizedBox();
              }),
          BlocBuilder<ChatDetailBloc, ChatDetailState>(
              bloc: _chatDetailBloc,
              builder: (_, state) {
                if (state.isEditKeyboard) {
                  Widget child = Text(
                    ChatHelper.decrypt(
                        _chatDetailBloc.conversationData![DB_KEY_ENCRYPT_KEY],
                        state.editDoc![DB_KEY_CONTENT]),
                    style: textStyleSmall.copyWith(
                        fontSize: 14.0, color: colorText),
                    overflow: TextOverflow.ellipsis,
                    maxLines: 1,
                  );
                  if (state.editDoc![DB_KEY_MESSAGE_TYPE] ==
                      MessageType.image.index) {
                    child = ClipRRect(
                      borderRadius: BorderRadius.circular(4),
                      child: Image.network(
                        state.editDoc![DB_KEY_CONTENT_URL],
                        height: 40,
                        fit: BoxFit.contain,
                      ),
                    );
                  }
                  return Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Row(
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          Icon(
                            Icons.edit,
                            size: 20,
                            color: colorText,
                          ),
                          const SizedBox(
                            width: 4,
                          ),
                          Text(
                            'reply'.tr,
                            style: textStyleSmall.copyWith(
                              color: colorText,
                              fontSize: 14.0,
                            ),
                          ),
                        ],
                      ),
                      const SizedBox(
                        height: 4,
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: hexColor('#131924'),
                            borderRadius: const BorderRadius.horizontal(
                                right: Radius.circular(16))),
                        child: Row(
                          children: [
                            Container(
                              height: 60,
                              width: 4.5,
                              margin: const EdgeInsets.only(right: 8),
                              decoration: BoxDecoration(
                                  color: colorText,
                                  borderRadius: const BorderRadius.horizontal(
                                      right: Radius.circular(12))),
                            ),
                            Expanded(
                              child: Container(
                                padding: const EdgeInsets.only(left: 16),
                                child: Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: [
                                    WidgetUserChatListBuilder(
                                        id: state.editDoc![DB_KEY_FROM],
                                        builder: (fullname, thumbnail) {
                                          return Text(
                                            state.editDoc![DB_KEY_FROM] ==
                                                    loggedFirebaseId
                                                ? 'you'.tr
                                                : fullname,
                                            overflow: TextOverflow.ellipsis,
                                            style: textStyleSmall.copyWith(
                                                fontWeight: FontWeight.bold,
                                                fontSize: 14),
                                          );
                                        }),
                                    const SizedBox(
                                      height: 8,
                                    ),
                                    child
                                  ],
                                ),
                              ),
                            ),
                            const SizedBox(
                              width: 16,
                            ),
                            InkWell(
                              onTap: () {
                                _chatDetailBloc
                                    .add(ChatDetailResetEditOrReplyEvent());
                              },
                              child: Icon(
                                Icons.cancel_rounded,
                                size: 32,
                                color: appColors.element,
                              ),
                            ),
                            const SizedBox(
                              width: 20,
                            ),
                          ],
                        ),
                      ),
                      const SizedBox(
                        height: 12,
                      ),
                    ],
                  );
                }
                return const SizedBox();
              }),
          Row(
            children: [
              BlocBuilder<ChatDetailBloc, ChatDetailState>(
                bloc: _chatDetailBloc,
                builder: (_, state) {
                  if (!state.isReplyKeyboard) {
                    return InkWell(
                      borderRadius: BorderRadius.circular(99),
                      onTap: () async {
                        hideKeyboard();
                        _chatDetailBloc
                            .add(ChatSendMessageEvent(type: MessageType.image));
                      },
                      child: Padding(
                        padding: const EdgeInsets.only(right: 16),
                        child: Icon(
                          Icons.camera_alt_outlined,
                          color: colorText,
                        ),
                      ),
                    );
                  }
                  return const SizedBox();
                },
              ),
              Expanded(
                child: Container(
                  decoration: BoxDecoration(
                      color: chatInputColor,
                      borderRadius: BorderRadius.circular(20)),
                  child: Stack(
                    alignment: Alignment.centerRight,
                    children: [
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Padding(
                            padding: const EdgeInsets.only(
                                left: 12, top: 8, bottom: 8),
                            child: Row(
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: [
                                Flexible(
                                  child: TextField(
                                    showCursor: true,
                                    maxLines: 8,
                                    minLines: 1,
                                    cursorColor: colorText,
                                    textCapitalization:
                                        TextCapitalization.sentences,
                                    style:
                                        robotoMedium.copyWith(color: colorText),
                                    controller: controller,
                                    decoration: InputDecoration(
                                      hoverColor: Colors.transparent,
                                      enabledBorder: _inputBorder,
                                      focusedBorder: _inputBorder,
                                      isCollapsed: true,
                                      border: _inputBorder,
                                      contentPadding:
                                          const EdgeInsets.fromLTRB(8, 0, 8, 0),
                                      hintText: 'typeamessage'.tr,
                                      hintStyle: robotoMedium.copyWith(
                                          fontStyle: FontStyle.italic,
                                          fontSize: 14),
                                    ),
                                  ),
                                ),
                                const SizedBox(
                                  width: 36,
                                )
                              ],
                            ),
                          ),
                        ],
                      ),
                      if (kIsWeb || Platform.isIOS || Platform.isAndroid)
                        ValueListenableBuilder(
                            valueListenable: controller,
                            builder: (_, value, child) {
                              if (controller.text.isEmpty) {
                                return InkWell(
                                  onTap: () async {
                                    AudioRecordResult? r =
                                        await showModalBottomSheet(
                                            clipBehavior: Clip.antiAlias,
                                            isScrollControlled: true,
                                            context: context,
                                            builder: (_) =>
                                                const WidgetAudioRecord());
                                    if (r != null) {
                                      _chatDetailBloc.add(ChatSendMessageEvent(
                                          type: MessageType.audio,
                                          imageExt: r.ext,
                                          imageFileName: r.fileName,
                                          imageFileMime: kIsWeb
                                              ? 'audio/webm'
                                              : 'audio/aac',
                                          imageData:
                                              await r.file.readAsBytes()));
                                    }
                                  },
                                  child: Container(
                                    width: 48,
                                    padding:
                                        const EdgeInsets.fromLTRB(8, 8, 8, 8),
                                    color: Colors.transparent,
                                    child: Center(
                                      child: Icon(Icons.mic,
                                          size: 20, color: colorText),
                                    ),
                                  ),
                                );
                              } else {
                                return GestureDetector(
                                  onTap: onSend,
                                  child: Container(
                                    width: 48,
                                    padding:
                                        const EdgeInsets.fromLTRB(8, 8, 8, 8),
                                    color: Colors.transparent,
                                    child: Center(
                                      child: Icon(Icons.send_rounded,
                                          size: 20, color: colorText),
                                    ),
                                  ),
                                );
                              }
                            })
                      else
                        GestureDetector(
                          onTap: onSend,
                          child: Container(
                            width: 48,
                            padding: const EdgeInsets.fromLTRB(8, 8, 8, 8),
                            color: Colors.transparent,
                            child: Center(
                              child: Icon(Icons.send_rounded,
                                  size: 20, color: colorText),
                            ),
                          ),
                        ),
                    ],
                  ),
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }
}
