import 'dart:math';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:_private_chat/chat_detail/widgets/widget_audio_record.dart';
import 'package:_private_core/_private_core.dart';

import 'widget_message_content.dart';

enum SoundPlaybackState { idle, preparing, playing, pause }

class WidgetMessageAudio extends StatefulWidget {
  final bool isMe;
  final String url;
  final Map data;
  const WidgetMessageAudio({
    Key? key,
    required this.url,
    required this.isMe,
    required this.data,
  }) : super(key: key);
  @override
  _WidgetMessageAudioState createState() => _WidgetMessageAudioState();
}

class _WidgetMessageAudioState extends State<WidgetMessageAudio> {
  List<int> listPreRandom = [];
  List<int> listAfterRandom = [];

  @override
  void initState() {
    super.initState();
    for (int i = 0; i < 15; i++) {
      listPreRandom.add(nextInt(1, 15));
      listAfterRandom.add(nextInt(1, 15));
    }
  }

  int nextInt(int min, int max) => min + Random().nextInt((max + 1) - min);

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (isMobile) {
      return GestureDetector(
        onTap: () {
          WidgetAudioRecord.playAudioUrl(widget.url);
        },
        child: Container(
          color: Colors.transparent,
          width: Get.width * .4,
          child: Row(children: [
            if (!widget.isMe)
              Container(
                height: 40,
                width: 3.5,
                decoration: BoxDecoration(
                    color: Colors.white,
                    borderRadius: BorderRadius.circular(4)),
              ),
            Expanded(
              child: Container(
                height: 32,
                padding: const EdgeInsets.only(top: 32 / 2, left: 4, right: 12),
                child: RepaintBoundary(
                  child: CustomPaint(
                    painter: SoundSpectrum(
                        listAfterRandom: listAfterRandom,
                        listPreRandom: listPreRandom,
                        processColor: Colors.transparent,
                        backGroundColor: hexColor('#385CF8'),
                        percent: 0),
                  ),
                ),
              ),
            ),
            if (widget.isMe)
              Container(
                height: 40,
                width: 3.5,
                decoration: BoxDecoration(
                    color: Colors.white,
                    borderRadius: BorderRadius.circular(4)),
              ),
          ]),
        ),
      );
    }
    return GestureDetector(
      onTap: () {
        WidgetAudioRecord.playAudioUrl(widget.url);
      },
      child: Container(
        color: Colors.transparent,
        width: Get.width / 3,
        child: Row(children: [
          Container(
            height: 135,
            width: 3.5,
            margin: const EdgeInsets.only(right: 8),
            decoration: BoxDecoration(
                color: appColors.text,
                borderRadius:
                    const BorderRadius.horizontal(right: Radius.circular(12))),
          ),
          Expanded(
            child: Container(
              height: 45,
              padding: const EdgeInsets.only(top: 22.5),
              child: RepaintBoundary(
                child: CustomPaint(
                  painter: SoundSpectrum(
                      listAfterRandom: listAfterRandom,
                      listPreRandom: listPreRandom,
                      processColor: Colors.transparent,
                      backGroundColor: hexColor('#385CF8'),
                      percent: 0),
                ),
              ),
            ),
          ),
        ]),
      ),
    );
  }
}
