import 'dart:async';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:_private_chat/widgets/photo_view_screen.dart';
import 'package:_private_core/_private_core.dart';
import 'package:_private_core/widgets/widgets.dart';
import 'dart:ui' as ui;

class WidgetMessageImage extends StatelessWidget {
  final Map data;
  const WidgetMessageImage({
    Key? key,
    required this.data,
  }) : super(key: key);

  Future<ui.Image> _ratioImage(image) async {
    late Completer<ui.Image> completer;
    completer = Completer<ui.Image>();
    image.image
        .resolve(const ImageConfiguration())
        .addListener(ImageStreamListener((ImageInfo info, bool _) {
      completer.complete(info.image);
    }));
    return await completer.future;
  }

  void openImage(i) {
    chatOptions!.chatOnOpenImage != null
        ? chatOptions!.chatOnOpenImage!(i, data)
        : Get.to(() => PhotoViewScreen(imageProvider: i), opaque: false);
  }

  double get limitHeight => 160;
  double get limitWidth => 160;

  @override
  Widget build(BuildContext context) {
    Image image = Image.network(data[DB_KEY_CONTENT_URL]);
    return FutureBuilder<ui.Image>(
      future: _ratioImage(image),
      builder: (BuildContext context, AsyncSnapshot<ui.Image> snapshot) {
        if (snapshot.hasData) {
          return GestureDetector(
            onTap: () {
              openImage(image.image);
            },
            child: snapshot.data!.width < snapshot.data!.height
                ? Container(
                    height: limitHeight,
                    width: snapshot.data!.width *
                        limitHeight /
                        snapshot.data!.height,
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(4),
                      image: DecorationImage(
                          image: image.image, fit: BoxFit.cover),
                    ),
                  )
                : Container(
                    height: snapshot.data!.height *
                        limitWidth /
                        snapshot.data!.width,
                    width: limitWidth,
                    decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(4),
                      image: DecorationImage(
                          image: image.image, fit: BoxFit.cover),
                    ),
                  ),
          );
        } else {
          return Shimmer.fromColors(
            baseColor: appColors.element,
            highlightColor: appColors.appBackground,
            child: Container(
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(8), color: Colors.black),
              height: limitHeight,
              width: limitWidth,
            ),
          );
        }
      },
    );
  }
}

class WidgetMessageImageGif extends StatelessWidget {
  final Map data;
  const WidgetMessageImageGif({
    Key? key,
    required this.data,
  }) : super(key: key);

  void openImage(i) {
    chatOptions!.chatOnOpenImage != null
        ? chatOptions!.chatOnOpenImage!(i, data)
        : Get.to(() => PhotoViewScreen(imageProvider: i), opaque: false);
  }

  double get limitHeight => 160;
  double get limitWidth => 160;

  @override
  Widget build(BuildContext context) {
    return ClipRRect(
      borderRadius: BorderRadius.circular(4),
      child: CachedNetworkImage(
        imageUrl: data[DB_KEY_CONTENT_URL],
        height: limitHeight,
        fit: BoxFit.fitHeight,
        errorWidget: (context, url, error) {
          return const SizedBox();
        },
        placeholder: (_, url) {
          return Shimmer.fromColors(
            baseColor: appColors.element,
            highlightColor: appColors.appBackground,
            child: Container(
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(8), color: Colors.black),
              height: limitHeight,
              width: limitWidth,
            ),
          );
        },
      ),
    );
  }
}
