import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:get/get.dart' as getx;
import 'package:get/get.dart';
import 'package:_private_chat/_private_chat.dart';
import 'package:_private_core/_private_core.dart';
import 'package:_private_core/widgets/widgets.dart';

import 'widget_more_button.dart';

class WidgetProfileChat extends StatefulWidget {
  final ChatDetailParams? params;
  final bool isEmbedWidget;
  final Widget Function(bool isEmbedWidget, dynamic state)? builder;
  const WidgetProfileChat({
    Key? key,
    this.params,
    this.isEmbedWidget = false,
    this.builder,
  }) : super(key: key);

  @override
  State<WidgetProfileChat> createState() => _WidgetProfileChatState();
}

class _WidgetProfileChatState extends State<WidgetProfileChat> {
  ChatDetailBloc bloc = Get.find<ChatDetailBloc>();
  late ChatDetailParams params;

  bool get isEmbedWidget => widget.isEmbedWidget;

  Color get colorText => Colors.white;

  bool isTyping = false;
  DateTime lastChange = DateTime.now();

  @override
  void initState() {
    super.initState();
    params = widget.params ?? getx.Get.arguments;
  }

  @override
  Widget build(BuildContext context) {
    if (chatOptions!.chatDetailProfileBuilder != null ||
        widget.builder != null) {
      return BlocBuilder<ChatDetailBloc, ChatDetailState>(
          bloc: bloc,
          builder: (_, state) {
            return KeyedSubtree(
              key: ValueKey(state.isLoading),
              child: widget.builder?.call(isEmbedWidget, state) ??
                  chatOptions!.chatDetailProfileBuilder!(isEmbedWidget, state),
            );
          });
    }
    return Padding(
      padding: const EdgeInsets.all(12.0),
      child: Row(
        children: [
          if (!isEmbedWidget)
            SizedBox(
              height: 60,
              child: Align(
                alignment: Alignment.centerLeft,
                child: IconButton(
                  onPressed: () => Get.back(),
                  icon: Icon(
                    Icons.arrow_back_ios_rounded,
                    size: 20,
                    color: colorText,
                  ),
                ),
              ),
            ),
          BlocBuilder<ChatListBloc, ChatListState>(
              bloc: Get.find<ChatListBloc>(),
              builder: (_, state) {
                return Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Hero(
                      tag: bloc.conversationId,
                      child: WidgetAvatar.withoutBorder(
                        radius: 28,
                        imageUrl: bloc.peerAvatar,
                      ),
                    ),
                    const SizedBox(
                      width: 16,
                    ),
                    Column(
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        Text(
                          bloc.peerName,
                          maxLines: 1,
                          style: robotoMedium.copyWith(
                              fontWeight: FontWeight.bold, color: colorText),
                        ),
                      ],
                    )
                  ],
                );
              }),
          const Spacer(),
          const WidgetMoreButtonChat(),
        ],
      ),
    );
  }
}
