import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:lottie/lottie.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core/_private_core.dart';
import 'package:_private_chat/widgets/widget_table_date_picker.dart';
import 'package:_private_chat/widgets/widget_time_picker.dart';
import 'package:_private_core/widgets/widgets.dart';

import '../bloc/chat_detail_bloc.dart';

enum _ContentType { date, time, property, none, propertySeleted }

class WidgetSendAppointment extends StatefulWidget {
  const WidgetSendAppointment({Key? key}) : super(key: key);
  @override
  _WidgetSendAppointmentState createState() => _WidgetSendAppointmentState();
}

class _WidgetSendAppointmentState extends State<WidgetSendAppointment> {
  final HomelidoRepo repo = HomelidoRepo();
  DateTime _time = DateTime.now();
  DateTime _date = DateTime.now();
  _ContentType _type = _ContentType.none;
  List<HomelidoProperty>? properties;
  HomelidoProperty? property;

  Color get textColor => appColors.text;

  @override
  void initState() {
    super.initState();
    _getProperties();
  }

  _getProperties() {
    repo.getPropertiesFromPro().then((value) {
      setState(() {
        properties = value;
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        if (_type == _ContentType.time || _type == _ContentType.date) {
          if (property != null) {
            setState(() {
              _type = _ContentType.propertySeleted;
            });
          } else {
            setState(() {
              _type = _ContentType.none;
            });
          }
        }
      },
      child: Container(
        width: Get.width,
        clipBehavior: Clip.hardEdge,
        decoration: BoxDecoration(
            color: hexColor('#273141'),
            borderRadius: const BorderRadius.only(
                topLeft: Radius.circular(32), topRight: Radius.circular(32))),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            const SizedBox(
              height: 16,
            ),
            Row(
              children: [
                const SizedBox(
                  width: 20,
                ),
                Text(
                  'meetingproposition'.tr,
                  textAlign: TextAlign.center,
                  style: robotoBold.copyWith(fontSize: 23, color: textColor),
                ),
                const Spacer(),
                Column(
                  children: [
                    Lottie.asset(
                      'assets/lotties/assist.json',
                      width: 36,
                      height: 36,
                    ),
                    const SizedBox(
                      height: 3,
                    ),
                    Text(
                      'arckipelassistant'.tr,
                      textAlign: TextAlign.center,
                      style: robotoMedium.copyWith(
                          fontSize: 8, color: Colors.white),
                    ),
                  ],
                ),
                const SizedBox(
                  width: 20,
                ),
              ],
            ),
            const SizedBox(
              height: 24,
            ),
            Container(
              height: 1,
              color: hexColor('#2D3A50'),
              width: Get.width,
            ),
            Container(
              padding: const EdgeInsets.symmetric(vertical: 12, horizontal: 24),
              child: Row(
                children: [
                  Text(
                    'date'.tr,
                    style:
                        robotoMedium.copyWith(color: textColor, fontSize: 15),
                  ),
                  Expanded(
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: [
                        GestureDetector(
                          onTap: () {
                            setState(() {
                              _type = _ContentType.date;
                            });
                          },
                          child: Container(
                            decoration: BoxDecoration(
                                borderRadius: BorderRadius.circular(6)),
                            padding: const EdgeInsets.symmetric(
                                vertical: 6, horizontal: 22),
                            child: Text(
                              _date.formatDate(),
                              style: robotoBold.copyWith(
                                  fontWeight: FontWeight.w500,
                                  color: Colors.white),
                            ),
                          ),
                        ),
                        GestureDetector(
                          onTap: () {
                            setState(() {
                              _type = _ContentType.time;
                            });
                          },
                          child: Container(
                            decoration: BoxDecoration(
                                borderRadius: BorderRadius.circular(6)),
                            padding: const EdgeInsets.symmetric(
                                vertical: 6, horizontal: 22),
                            child: Text(
                              _time.formatTime(),
                              style: robotoBold.copyWith(
                                  fontWeight: FontWeight.w500,
                                  color: Colors.white),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
            Container(
              height: 1,
              color: hexColor('#2D3A50'),
              width: Get.width,
            ),
            GestureDetector(
              onTap: _type == _ContentType.property
                  ? () {
                      setState(() {
                        _type = _ContentType.none;
                      });
                    }
                  : null,
              child: Container(
                color: Colors.transparent,
                padding:
                    const EdgeInsets.symmetric(vertical: 12, horizontal: 24),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    Text(
                      'property'.tr,
                      style:
                          robotoMedium.copyWith(color: textColor, fontSize: 15),
                    ),
                    if (_type == _ContentType.propertySeleted)
                      Expanded(
                        child: Align(
                          alignment: Alignment.centerRight,
                          child: GestureDetector(
                            onTap: () {
                              setState(() {
                                _type = _ContentType.none;
                                property = null;
                              });
                              _getProperties();
                            },
                            child: Container(
                              padding: const EdgeInsets.all(2),
                              child: Icon(
                                Icons.close_rounded,
                                color: textColor,
                                size: 20,
                              ),
                              decoration: BoxDecoration(
                                  shape: BoxShape.circle,
                                  color: Colors.transparent,
                                  border: Border.all(
                                      color: textColor, width: 1.65)),
                            ),
                          ),
                        ),
                      )
                    else if (_type == _ContentType.none)
                      Expanded(
                        child: Center(
                          child: GestureDetector(
                            onTap: () {
                              if (_type != _ContentType.property) {
                                setState(() {
                                  _type = _ContentType.property;
                                });
                                _getProperties();
                              } else {
                                setState(() {
                                  _type = _ContentType.none;
                                });
                                _getProperties();
                              }
                            },
                            child: Container(
                              padding: const EdgeInsets.symmetric(
                                  vertical: 6, horizontal: 22),
                              child: Text(
                                properties != null && properties!.isEmpty
                                    ? 'youdonnothaveanyproperty'
                                    : 'selectaproperty'.tr,
                                style: robotoBold.copyWith(
                                    fontWeight: FontWeight.w500,
                                    fontStyle: FontStyle.italic,
                                    fontSize: 15,
                                    color: textColor),
                              ),
                            ),
                          ),
                        ),
                      )
                    else
                      const Spacer(),
                    if (_type != _ContentType.propertySeleted)
                      Icon(
                        _type == _ContentType.property
                            ? Icons.keyboard_arrow_up
                            : Icons.keyboard_arrow_down,
                        color: Colors.white,
                        size: 20,
                      ),
                  ],
                ),
              ),
            ),
            if (_type != _ContentType.propertySeleted &&
                _type != _ContentType.property)
              Container(
                height: 1,
                color: hexColor('#2D3A50'),
                width: Get.width,
              ),
            _buildBody(),
            Container(
              padding: EdgeInsets.fromLTRB(
                  24, 8, 24, 16 + MediaQuery.of(context).padding.bottom),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  GestureDetector(
                    onTap: () {
                      if (_type == _ContentType.propertySeleted ||
                          property != null) {
                        setState(() {
                          _type = _ContentType.none;
                          property = null;
                        });
                      } else {
                        Get.find<ChatDetailBloc>().add(
                            ChatDetailUpdateSentAppointmentStateEvent(false));
                      }
                    },
                    child: Container(
                      width: Get.width / 2.5,
                      decoration: BoxDecoration(
                          border: Border.all(color: textColor, width: 1.2),
                          borderRadius: BorderRadius.circular(12)),
                      padding: const EdgeInsets.symmetric(vertical: 10),
                      child: Center(
                        child: Text(
                          'cancel2'.tr,
                          style: robotoBold.copyWith(color: textColor),
                        ),
                      ),
                    ),
                  ),
                  Opacity(
                    opacity: properties == null ||
                            properties!.isEmpty ||
                            property == null
                        ? .35
                        : 1,
                    child: IgnorePointer(
                      ignoring: properties == null ||
                          properties!.isEmpty ||
                          property == null,
                      child: GestureDetector(
                        onTap: () {
                          Get.find<ChatDetailBloc>().add(
                              ChatDetailSentAppointmentEvent(
                                  DateTime(_date.year, _date.month, _date.day,
                                      _time.hour, _time.minute),
                                  property!));
                        },
                        child: Container(
                          width: Get.width / 2.5,
                          decoration: BoxDecoration(
                              color: textColor,
                              border: Border.all(color: textColor, width: 1.2),
                              borderRadius: BorderRadius.circular(12)),
                          padding: const EdgeInsets.symmetric(vertical: 10),
                          child: Center(
                            child: Row(
                              mainAxisSize: MainAxisSize.min,
                              children: [
                                Text(
                                  'send2'.tr,
                                  style:
                                      robotoBold.copyWith(color: Colors.white),
                                ),
                                const SizedBox(
                                  width: 6,
                                ),
                                // SvgPicture.asset(
                                //   'assets/images/svg/ic_send.svg',
                                //   width: 14,
                                // )
                              ],
                            ),
                          ),
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget _buildBody() {
    double height = 0;
    Widget child = const SizedBox();
    switch (_type) {
      case _ContentType.propertySeleted:
        HomelidoProperty m = property!;
        height = 100;
        child = Container(
          padding: const EdgeInsets.fromLTRB(24, 0, 24, 8),
          child: Row(
            children: [
              WidgetAppImage(
                imageUrl:
                    appImageCorrectUrl(m.image!, base: arckipelbaseUrlHomelido),
                width: 115,
                height: 80,
                boxFit: BoxFit.cover,
                radius: 12,
              ),
              const SizedBox(
                width: 24,
              ),
              Expanded(
                child: SizedBox(
                  height: 80,
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        m.title ?? '',
                        maxLines: 1,
                        overflow: TextOverflow.ellipsis,
                        style: robotoBold.copyWith(color: Colors.white),
                      ),
                      const SizedBox(
                        height: 6,
                      ),
                      Row(
                        children: [
                          const Icon(
                            Icons.location_on,
                            size: 18,
                            color: Colors.white,
                          ),
                          const SizedBox(
                            width: 8,
                          ),
                          Expanded(
                            child: Text(
                              m.address ?? '',
                              maxLines: 1,
                              overflow: TextOverflow.ellipsis,
                              style:
                                  textStyleSmall.copyWith(color: Colors.white),
                            ),
                          ),
                        ],
                      ),
                      const SizedBox(
                        height: 6,
                      ),
                      Row(
                        crossAxisAlignment: CrossAxisAlignment.end,
                        children: [
                          Text(
                            m.salePrice != null
                                ? '${m.salePrice} ${currencySymbol(m.currency)}'
                                : '${m.price} ${currencySymbol(m.currency)}/${'month'.tr}',
                            style: textStyleSmall.copyWith(
                                color: appColors.redHomelido, fontSize: 9),
                          ),
                          const SizedBox(
                            width: 24,
                          ),
                          const Icon(
                            Icons.aspect_ratio,
                            size: 14,
                            color: Colors.white,
                          ),
                          const SizedBox(
                            width: 4,
                          ),
                          Text(
                            m.area == null || m.areaUnit == null
                                ? 'unknow'.tr
                                : '${m.area} ${m.areaUnit}',
                            style: textStyleSmall.copyWith(
                                color: Colors.white, fontSize: 9),
                          ),
                          const Spacer(),
                          InkWell(
                            onTap: () {
                              // showMaterialModalBottomSheet(
                              //     backgroundColor: Colors.transparent,
                              //     context: Get.context!,
                              //     builder: (_) => HomelidoScreen(
                              //         m: m.homelidoPropertyInfo!));
                            },
                            child: Text(
                              'more...'.tr,
                              style: textStyleSmall.copyWith(
                                  color: textColor, fontSize: 8.5),
                            ),
                          ),
                        ],
                      )
                    ],
                  ),
                ),
              )
            ],
          ),
        );
        break;

      case _ContentType.date:
        height = 300;
        child = Padding(
          padding: const EdgeInsets.symmetric(horizontal: 24),
          child: WidgetTableDatePicker(
            onTimeChange: (DateTime time) {
              setState(() {
                _date = time;
              });
            },
          ),
        );
        break;
      case _ContentType.time:
        height = 200;
        child = Padding(
          padding: EdgeInsets.symmetric(horizontal: Get.width / 5),
          child: WidgetTimePicker(
            onTimeChange: (DateTime time) {
              setState(() {
                _time = time;
              });
            },
          ),
        );
        break;
      case _ContentType.property:
        height = 335;
        if (properties == null) {
          child = const WidgetLoadingCupertino();
          height = 150;
        } else if (properties!.isEmpty) {
          height = 10;
          child = const SizedBox();
        } else {
          child = ListView.builder(
              padding: EdgeInsets.zero,
              scrollDirection: Axis.vertical,
              itemCount: properties!.length,
              itemBuilder: (_, index) {
                HomelidoProperty m = properties![index];
                return GestureDetector(
                  onTap: () {
                    setState(() {
                      _type = _ContentType.propertySeleted;
                      property = m;
                    });
                  },
                  child: Container(
                    color: Colors.transparent,
                    padding: const EdgeInsets.fromLTRB(24, 12, 24, 16),
                    child: Row(
                      children: [
                        WidgetAppImage(
                          imageUrl: appImageCorrectUrl(m.image!,
                              base: arckipelbaseUrlHomelido),
                          width: 115,
                          height: 80,
                          boxFit: BoxFit.cover,
                          radius: 12,
                        ),
                        const SizedBox(
                          width: 24,
                        ),
                        Expanded(
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text(
                                m.title ?? '',
                                maxLines: 1,
                                overflow: TextOverflow.ellipsis,
                                style: robotoBold.copyWith(color: Colors.white),
                              ),
                              const SizedBox(
                                height: 6,
                              ),
                              Row(
                                children: [
                                  const Icon(
                                    Icons.location_on,
                                    size: 18,
                                    color: Colors.white,
                                  ),
                                  const SizedBox(
                                    width: 8,
                                  ),
                                  Expanded(
                                    child: Text(
                                      m.address ?? '',
                                      maxLines: 1,
                                      overflow: TextOverflow.ellipsis,
                                      style: textStyleSmall.copyWith(
                                          color: Colors.white),
                                    ),
                                  ),
                                ],
                              ),
                              const SizedBox(
                                height: 6,
                              ),
                              Row(
                                crossAxisAlignment: CrossAxisAlignment.end,
                                children: [
                                  Text(
                                    m.salePrice != null
                                        ? '${m.salePrice} ${currencySymbol(m.currency)}'
                                        : '${m.price} ${currencySymbol(m.currency)}/${'month'.tr}',
                                    style: textStyleSmall.copyWith(
                                        color: appColors.redHomelido,
                                        fontSize: 9),
                                  ),
                                  const SizedBox(
                                    width: 24,
                                  ),
                                  const Icon(
                                    Icons.aspect_ratio,
                                    size: 14,
                                    color: Colors.white,
                                  ),
                                  const SizedBox(
                                    width: 4,
                                  ),
                                  Text(
                                    m.area == null || m.areaUnit == null
                                        ? 'unknow'.tr
                                        : '${m.area} ${m.areaUnit}',
                                    style: textStyleSmall.copyWith(
                                        color: Colors.white, fontSize: 9),
                                  ),
                                  const Spacer(),
                                  InkWell(
                                    onTap: () {
                                      // showMaterialModalBottomSheet(
                                      //     backgroundColor: Colors.transparent,
                                      //     context: Get.context!,
                                      //     builder: (_) => HomelidoScreen(
                                      //         m: m.homelidoPropertyInfo!));
                                    },
                                    child: Text(
                                      'more...'.tr,
                                      style: textStyleSmall.copyWith(
                                          color: textColor, fontSize: 8.5),
                                    ),
                                  ),
                                ],
                              )
                            ],
                          ),
                        )
                      ],
                    ),
                  ),
                );
              });
        }
        break;
      default:
        break;
    }
    return AnimatedContainer(
      duration: const Duration(milliseconds: 350),
      width: Get.width,
      height: height,
      child: child,
    );
  }
}
