import 'dart:async';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:_private_chat/utils/utils.dart';
import 'package:_private_core/_private_core.dart';

part 'connection_status_event.dart';
part 'connection_status_state.dart';

class ConnectionStatusBloc
    extends Bloc<ConnectionStatusEvent, ConnectionStatusState> {
  ConnectionStatusBloc()
      : super(ConnectionStatusState(
          connectionStatusByKeys: {},
          users: [],
          blocked: [],
          muted: [],
        )) {
    on<ConnectionStatusInitEvent>(_onInit);
    on<ConnectionStatusUpdateEvent>(_onUpdate);
    on<ConnectionStatusSetStatusEvent>(_onSetStatus);
    on<ConnectionStatusBlockToEvent>(_onBlockTo);
    on<ConnectionStatusMuteToEvent>(_onMuteTo);
  }

  late final CollectionReference _col =
      FirebaseFirestore.instance.collection(C_CONNECTION_STATUS);
  StreamSubscription? _subscription;

  static setStatusReceived(firebaseId) async {
    FirebaseFirestore.instance
        .collection(C_CONNECTION_STATUS)
        .doc(firebaseId)
        .update(
            {DB_KEY_PERSONAL_RECEIVED: DateTime.now().millisecondsSinceEpoch});
  }

  _dataToState(QuerySnapshot query) {
    Map connectionStatusByKeys = {};
    query.docs.forEach((d) {
      Map data = d.data() as Map;
      if (data[DB_KEY_FIREBASE_ID] == loggedFirebaseId) {
        add(ConnectionStatusUpdateEvent(
            muted: data[DB_KEY_MUTED], blocked: data[DB_KEY_BLOCKED]));
      } else if (state.users.contains(data[DB_KEY_FIREBASE_ID])) {
        connectionStatusByKeys[data[DB_KEY_FIREBASE_ID]] = data;
        add(ConnectionStatusUpdateEvent(
            connectionStatusByKeys: connectionStatusByKeys));
      }
    });
  }

  _onMuteTo(ConnectionStatusMuteToEvent event, emit) async {
    Map<String, dynamic> data = {};
    if (event.status) {
      data.addAll({
        DB_KEY_MUTED: List.from(state.muted + [event.id])
      });
    } else {
      data.addAll(
          {DB_KEY_MUTED: List.from(state.muted.where((e) => e != event.id))});
    }

    _col.doc(loggedFirebaseId).set(data, SetOptions(merge: true));
  }

  _onBlockTo(ConnectionStatusBlockToEvent event, emit) async {
    Map<String, dynamic> data = {};
    if (event.status) {
      data.addAll({
        DB_KEY_BLOCKED: List.from(state.blocked + [event.id])
      });
    } else {
      data.addAll({
        DB_KEY_BLOCKED: List.from(state.blocked.where((e) => e != event.id))
      });
    }

    _col.doc(loggedFirebaseId).set(data, SetOptions(merge: true));
  }

  _onSetStatus(ConnectionStatusSetStatusEvent event, emit) async {
    Map<String, dynamic> data = {
      DB_KEY_FIREBASE_ID: loggedFirebaseId,
      DB_KEY_PERSONAL_RECEIVED: DateTime.now().millisecondsSinceEpoch
    };
    if (event.isOnline != null) {
      if (event.isOnline == true) {
        data[DB_KEY_PERSONAL] = true;
      } else {
        data[DB_KEY_PERSONAL] = DateTime.now().millisecondsSinceEpoch;
      }
      data[DB_KEY_CONVERSATION] = ConversationStatus.none.name;
      data[DB_KEY_CONVERSATION_ID] = '';
    } else if (event.conversationStatus != null) {
      data[DB_KEY_PERSONAL] = true;
      data[DB_KEY_CONVERSATION] = event.conversationStatus!.name;
      if (event.conversationStatus == ConversationStatus.none) {
        data[DB_KEY_CONVERSATION_ID] = '';
      } else {
        data[DB_KEY_CONVERSATION_ID] = event.conversationId;
      }
    }

    _col.doc(loggedFirebaseId).set(data, SetOptions(merge: true));
  }

  _onUpdate(ConnectionStatusUpdateEvent event, emit) async {
    emit(state.update(
      connectionStatusByKeys: event.connectionStatusByKeys,
      muted: event.muted,
      blocked: event.blocked,
    ));
  }

  _onInit(ConnectionStatusInitEvent event, emit) async {
    if (state.users != event.users) {
      emit(state.update(users: event.users));
      var query = await _col
          // .where(DB_KEY_FIREBASE_ID, whereIn: state.users)
          .get();
      _dataToState(query);

      _subscription?.cancel();
      _subscription = _col
          // .where(DB_KEY_FIREBASE_ID, whereIn: state.users)
          .snapshots(includeMetadataChanges: true)
          .listen((query) {
        _dataToState(query);
      });
    }
  }
}
