import 'package:flutter/material.dart';
import 'package:_private_core/_private_core.dart';
import 'package:syncfusion_flutter_core/theme.dart';
import 'package:syncfusion_flutter_datepicker/datepicker.dart';

class WidgetTableDatePicker extends StatefulWidget {
  final Function(DateTime) onTimeChange;
  final DateTime? initialSelectedDate;
  const WidgetTableDatePicker(
      {Key? key, required this.onTimeChange, this.initialSelectedDate})
      : super(key: key);

  @override
  _WidgetTableDatePickerState createState() => _WidgetTableDatePickerState();
}

class _WidgetTableDatePickerState extends State<WidgetTableDatePicker> {
  TextStyle get textStyle => robotoMedium.copyWith(color: Colors.white);
  @override
  Widget build(BuildContext context) {
    return Container(
      margin: const EdgeInsets.only(top: 20, bottom: 12),
      clipBehavior: Clip.hardEdge,
      padding: const EdgeInsets.all(8.0),
      decoration: BoxDecoration(
        color: appColors.appBGColor,
        borderRadius: BorderRadius.circular(20),
      ),
      child: SfTheme(
        data: SfThemeData(
          dateRangePickerThemeData: SfDateRangePickerThemeData(
            leadingDatesTextStyle: textStyle,
            rangeSelectionTextStyle: textStyle,
            todayTextStyle: textStyle,
            weekNumberTextStyle: textStyle,
            weekendDatesTextStyle: textStyle,
            specialDatesTextStyle: textStyle,
            blackoutDatesTextStyle: textStyle,
            leadingCellTextStyle: textStyle,
            todayCellTextStyle: textStyle,
            disabledCellTextStyle: textStyle,
            disabledDatesTextStyle: textStyle,
            trailingDatesTextStyle: textStyle,
            headerTextStyle: textStyle,
            cellTextStyle: textStyle,
            selectionTextStyle: textStyle,
            viewHeaderTextStyle: textStyle,
            // activeDatesTextStyle: textStyle,
            // backgroundColor: AppColors.appBGColor,
            // selectionColor: Colors.red,
            // startRangeSelectionColor: Colors.red,
            // endRangeSelectionColor: Colors.red,
            // headerBackgroundColor: Colors.red,
            // viewHeaderBackgroundColor: Colors.red,
            // todayHighlightColor: Colors.red,
            // rangeSelectionColor: Colors.red,
            // weekNumberBackgroundColor: Colors.red,
          ),
        ),
        child: SfDateRangePicker(
          enablePastDates: true,
          allowViewNavigation: false,
          initialSelectedDate: widget.initialSelectedDate,
          selectionColor: Colors.blue,
          selectionTextStyle: textStyle,
          yearCellStyle: DateRangePickerYearCellStyle(textStyle: textStyle),
          headerStyle: DateRangePickerHeaderStyle(textStyle: textStyle),
          monthCellStyle: DateRangePickerMonthCellStyle(textStyle: textStyle),
          onSelectionChanged: (DateRangePickerSelectionChangedArgs args) {
            widget.onTimeChange(args.value);
          },
          selectionMode: DateRangePickerSelectionMode.single,
        ),
      ),
    );
  }
}
